# $Id: 01_start_and_end_handlers.t,v 1.3 2003/08/13 15:55:51 mrodrigu Exp $

use strict;
use warnings;

#########################

# change 'tests => 1' to 'tests => last_test_to_print';

use Test::More qw(no_plan);

my @possible_parsers= qw( XML::SAX::PurePerl XML::Parser::PerlSAX XML::LibXML::SAX::Parser XML::SAX::Expat);
my @parsers;

my $test_all= -f "test_all"; 

foreach my $parser ( @possible_parsers)
  { if( eval "require $parser")
    { eval "import $parser";# or die "hu-oh, error importing $parser even though the require succeded: $@"; 
      push @parsers, $parser;
    }
  }

if( @parsers) 
  { diag "testing with " . join( ", ", @parsers) . "\n"; }
else
  { die "you need to have a SAX parser installed (one of " . join( ", ", @possible_parsers) . ") none found"; }

BEGIN { use_ok( 'XML::SAX::Context'); }


my @doc;

{ local $/="\n\n";
  @doc= <DATA>;
} 


my %parse_string=( 'XML::Parser::PerlSAX' => 'parse');

foreach my $parser( @parsers)
  { my $parse_string= $parse_string{$parser} || 'parse_string';
    my $test= qq{ sub { my \$doc= shift;
                        my \$handler= TestHandler\->new;
                        \$handler->{my_parser_name}= '$parser';
                        my \$parser= $parser\->new(Handler => \$handler);
                        \$parser->$parse_string( \$doc);
                      }
                };
    my $sub_test= eval "$test";
    foreach my $doc (@doc)
      { if( $doc=~ /__INTERNAL__/) { if( $test_all) { next; } else { last; } }
        $sub_test->( $doc);
      }
  }

exit;


package TestHandler;
use YAML;
use Test::More;

use base qw(XML::SAX::Context);

#sub diag { die @_ };

# globals so even characters can access them
my $expected_current;
my @expected_context;
my $expected_depth;
my $expected_parent_name;

sub start_document
  { $expected_current='';
    @expected_context=();
    $expected_depth=0;
    $expected_parent_name='';
  }


sub start_element
  { my( $self, $elt)= @_;
    $expected_current= $elt->{Name};
    $expected_parent_name= $expected_context[-1] || '';
    push @expected_context, $elt->{Name};
    $expected_depth++;
    is( $self->depth, $expected_depth, "depth (start)") or diag() ;
    is( $self->current_name, $expected_current, "current_name (start)")
      or diag( "context is: ", Dump( $self->{_context}), "\nexpected_current= $expected_current\ncurrent found: ", $self->current_name, "\n") ;
    is( stringify( $self->ancestor_names), stringify( @expected_context), "ancestor_names (start)") or diag();
    
    is( $self->parent_name, $expected_parent_name, "parent_name (start)") or diag();
    if( $expected_parent_name) { ok( $self->in_element( $expected_parent_name)) or diag(); }
  }

sub end_element
  { my( $self, $elt)= @_;
    $expected_current= $elt->{Name};
    $expected_parent_name= $expected_context[-2] || '';
    is( $self->depth, $expected_depth, "depth (end)") or diag();
    is( $self->current_name, $expected_current, "current_name (end)") or diag();
    is( stringify( $self->ancestor_names), stringify( @expected_context), "ancestor_names (end)") or diag();
    is( $self->parent_name, $expected_parent_name, "parent_name (end)") or diag();
    $expected_depth--;
    pop @expected_context;
    $expected_current= $expected_context[-1];
  }

sub characters
  { my( $self, $chars)= @_;
    is( $self->depth, $expected_depth, "depth (char)") or diag( $self->{my_parser_name}, "  - char");
    is( $self->current_name, $expected_current, "current_name (char)")
      or diag( "current_name (char)\nchar: /$chars->{Data}/\ncontext is: \n", Dump( $self->{_context}), "\nexpected_current= $expected_current\ncurrent found: ", $self->current_name, "\n") ;
    is( stringify( $self->ancestor_names), stringify( @expected_context), "ancestor_names (char)") or diag();
    #is( $self->parent_name, $expected_parent_name, "parent_name (char)") or diag();
    ok( $self->in_element( $expected_current)) or diag();
    
  }

sub stringify { return join ':', @_ }

package main;

# one empty line between docs
# add text pop after closing elements so the test can make sense of it

__DATA__
<doc><elt1>text</elt1></doc>

<doc>
  <elt1 id="elt1-1">
    <elt2 id="elt2-1">This is an elt2 element (1)</elt2>
    <elt2 id="elt2-2">This is an elt2 element (2)</elt2>
    <elt2 id="elt2-3">This is an elt2 element (3)</elt2>
  </elt1>
  <elt1 id="elt2-1">
    <elt2 id="elt2-4">This is an elt2 element (6)</elt2>
    <elt2 id="elt2-5">This is an elt2 element (5)</elt2>
    <elt2 id="elt2-6">This is an elt2 element (6)</elt2>
  </elt1>
</doc>

<doc>foo <elt>bar</elt>pop</doc>

<doc>foo <elt />pop</doc>

__INTERNAL__

<?xml version="1.0" encoding="ISO-8859-1"?>
<Finvoice Version="1.0">
	<SellerPartyDetails>
		<SellerPartyIdentifier>0123456-7</SellerPartyIdentifier>
		<SellerOrganisationName>Pullin Kala Oy</SellerOrganisationName>
		<SellerOrganisationTaxCode>0123456-7</SellerOrganisationTaxCode>
		<SellerPostalAddressDetails>
			<SellerStreetName>Haapatie 7</SellerStreetName>
			<SellerTownName>Helsinki</SellerTownName>
			<SellerPostCodeIdentifier>00100</SellerPostCodeIdentifier>
			<CountryCode>FI</CountryCode>
			<CountryName>FINLAND</CountryName>
			<SellerPostOfficeBoxIdentifier>PL 302</SellerPostOfficeBoxIdentifier>
		</SellerPostalAddressDetails>
	</SellerPartyDetails>
	<SellerContactPersonName>Hanna Paananen</SellerContactPersonName>
	<SellerCommunicationDetails>
		<SellerPhoneNumberIdentifier>050-5432659</SellerPhoneNumberIdentifier>
		<SellerEmailaddressIdentifier>hanna.paananen@pullinkala.fi</SellerEmailaddressIdentifier>
	</SellerCommunicationDetails>
	<SellerInformationDetails>
		<SellerHomeTownName>Helsinki</SellerHomeTownName>
		<SellerVatRegistrationText>Alv.Rek</SellerVatRegistrationText>
		<SellerVatRegistrationDate Format="CCYYMMDD">19990321</SellerVatRegistrationDate>
		<SellerPhoneNumber>(09) 542 1222</SellerPhoneNumber>
		<SellerFaxNumber>(09) 542 2221</SellerFaxNumber>
		<SellerCommonEmailaddressIdentifier>palaute@pullinkala.fi</SellerCommonEmailaddressIdentifier>
		<SellerWebaddressIdentifier>www.pullinkala.fi</SellerWebaddressIdentifier>
		<SellerAccountDetails>
			<SellerAccountID IdentificationSchemeName="IBAN">FI1234567890123456</SellerAccountID>
			<SellerBic IdentificationSchemeName="BIC">OKOYFIHH</SellerBic>
		</SellerAccountDetails>
		<SellerAccountDetails>
			<SellerAccountID IdentificationSchemeName="IBAN">FI1234567890123456</SellerAccountID>
			<SellerBic IdentificationSchemeName="BIC">NDEAFIHH</SellerBic>
		</SellerAccountDetails>
	</SellerInformationDetails>
	<BuyerPartyDetails>
		<BuyerPartyIdentifier>CL12221</BuyerPartyIdentifier>
		<BuyerOrganisationName>Kalakauppa Vilkkunen</BuyerOrganisationName>
		<BuyerPostalAddressDetails>
			<BuyerStreetName>Haapatie 1</BuyerStreetName>
			<BuyerTownName>Helsinki</BuyerTownName>
			<BuyerPostCodeIdentifier>00211</BuyerPostCodeIdentifier>
			<CountryCode/>
			<CountryName/>
			<BuyerPostOfficeBoxIdentifier/>
		</BuyerPostalAddressDetails>
	</BuyerPartyDetails>
	<BuyerContactPersonName>Maija Vikkunen</BuyerContactPersonName>
	<BuyerCommunicationDetails>
		<BuyerPhoneNumberIdentifier>(09)221222</BuyerPhoneNumberIdentifier>
		<BuyerEmailaddressIdentifier>maija.vilkkunen@kolumbus.fi</BuyerEmailaddressIdentifier>
	</BuyerCommunicationDetails>
	<InvoiceDetails>
		<InvoiceTypeCode>INV01</InvoiceTypeCode>
		<InvoiceTypeText>LASKU</InvoiceTypeText>
		<OriginCode>Origin</OriginCode>
		<InvoiceNumber>1/2002</InvoiceNumber>
		<InvoiceDate Format="CCYYMMDD">20020612</InvoiceDate>
		<SellerReferenceIdentifier>212</SellerReferenceIdentifier>
		<OrderIdentifier>TI0988</OrderIdentifier>
		<InvoiceTotalVatExcludedAmount AmountCurrencyIdentifier="EUR">100,00</InvoiceTotalVatExcludedAmount>
		<InvoiceTotalVatAmount AmountCurrencyIdentifier="EUR">22,00</InvoiceTotalVatAmount>
		<InvoiceTotalVatIncludedAmount AmountCurrencyIdentifier="EUR">122,00</InvoiceTotalVatIncludedAmount>
		<ShortProposedAccountIdentifier>9922</ShortProposedAccountIdentifier>
		<NormalProposedAccountIdentifier>8822</NormalProposedAccountIdentifier>
		<AccountDimensionText>4500</AccountDimensionText>
		<VatSpecificationDetails>
			<VatBaseAmount AmountCurrencyIdentifier="EUR">100,00</VatBaseAmount>
			<VatRatePercent>22</VatRatePercent>
			<VatRateAmount AmountCurrencyIdentifier="EUR">22,00</VatRateAmount>
		</VatSpecificationDetails>
		<PaymentTermsDetails>
			<PaymentTermsFreeText>14 piv netto</PaymentTermsFreeText>
			<PaymentOverDueFineDetails>
				<PaymentOverDueFineFreeText>Yliaikakorko 16%</PaymentOverDueFineFreeText>
				<PaymentOverDueFinePercent>16</PaymentOverDueFinePercent>
			</PaymentOverDueFineDetails>
		</PaymentTermsDetails>
	</InvoiceDetails>
	<PaymentStatusDetails>
		<PaymentStatusCode>NOTPAID</PaymentStatusCode>
	</PaymentStatusDetails>
	<VirtualBankBarcode>250003210002229000122000000000000000862074102062800009</VirtualBankBarcode>
	<InvoiceRow>
		<ArticleIdentifier>123213123332123</ArticleIdentifier>
		<ArticleName>TUURA</ArticleName>
		<BuyerArticleIdentifier>1232321232332</BuyerArticleIdentifier>
		<DeliveredQuantity QuantityUnitCode="Kpl">10</DeliveredQuantity>
		<OrderedQuantity QuantityUnitCode="Kpl">10</OrderedQuantity>
		<UnitPriceAmount AmountCurrencyIdentifier="EUR" UnitPriceUnitCode="e/kpl">5,00</UnitPriceAmount>
		<RowIdentifier>221</RowIdentifier>
		<RowIdentifierDate Format="CCYYMMDD">20020611</RowIdentifierDate>
		<RowDeliveryDate Format="CCYYMMDD">20020615</RowDeliveryDate>
		<RowShortProposedAccountIdentifier>9922</RowShortProposedAccountIdentifier>
		<RowNormalProposedAccountIdentifier>8822</RowNormalProposedAccountIdentifier>
		<RowAccountDimensionText>4500</RowAccountDimensionText>
		<RowFreeText>Tuurat myydn varsien kanssa.</RowFreeText>
		<RowDiscountPercent>0</RowDiscountPercent>
		<RowVatRatePercent>22</RowVatRatePercent>
		<RowVatAmount AmountCurrencyIdentifier="EUR">11,00</RowVatAmount>
		<RowVatExcludedAmount AmountCurrencyIdentifier="EUR">50,00</RowVatExcludedAmount>
		<RowAmount AmountCurrencyIdentifier="EUR">61,00</RowAmount>
	</InvoiceRow>
	<InvoiceRow>
		<ArticleIdentifier>123213123332123</ArticleIdentifier>
		<ArticleName>HUOLLON VUOSIMAKSU</ArticleName>
		<StartDate Format="CCYYMMDDHHMMSS">20020611000000</StartDate>
		<EndDate Format="CCYYMMDDHHMMSS">2003062011000000</EndDate>
		<RowShortProposedAccountIdentifier>9923</RowShortProposedAccountIdentifier>
		<RowNormalProposedAccountIdentifier>8823</RowNormalProposedAccountIdentifier>
		<RowAccountDimensionText>4500</RowAccountDimensionText>
		<RowVatRatePercent>22</RowVatRatePercent>
		<RowVatAmount AmountCurrencyIdentifier="EUR">11,00</RowVatAmount>
		<RowVatExcludedAmount AmountCurrencyIdentifier="EUR">50,00</RowVatExcludedAmount>
		<RowAmount AmountCurrencyIdentifier="EUR">61,00</RowAmount>
	</InvoiceRow>
	<EpiDetails>
		<EpiIdentificationDetails>
			<EpiDate Format="CCYYMMDD">20020612</EpiDate>
		</EpiIdentificationDetails>
		<EpiPartyDetails>
			<EpiBfiPartyDetails>
				<EpiBfiIdentifier IdentificationSchemeName="BIC">OKOYFIHH</EpiBfiIdentifier>
			</EpiBfiPartyDetails>
			<EpiBeneficiaryPartyDetails>
				<EpiNameAddressDetails>Pullin Kala</EpiNameAddressDetails>
				<EpiAccountID IdentificationSchemeName="IBAN">FI1234567890123456</EpiAccountID>
			</EpiBeneficiaryPartyDetails>
		</EpiPartyDetails>
		<EpiPaymentInstructionDetails>
			<EpiRemittanceInfoIdentifier>8620741</EpiRemittanceInfoIdentifier>
			<EpiInstructedAmount AmountCurrencyIdentifier="EUR">122,00</EpiInstructedAmount>
			<EpiCharge ChargeOption="SHA">SHA</EpiCharge>
			<EpiDateOptionDate Format="CCYYMMDD">20020628</EpiDateOptionDate>
		</EpiPaymentInstructionDetails>
	</EpiDetails>
</Finvoice>
