# $Id: 02_no_end_handler.t,v 1.3 2003/08/18 13:49:16 mrodrigu Exp $

use strict;

#########################

# change 'tests => 1' to 'tests => last_test_to_print';

use Test::More qw(no_plan);

my @possible_parsers= qw( XML::SAX::PurePerl XML::Parser::PerlSAX XML::LibXML::SAX::Parser XML::SAX::Expat);
my @parsers;

foreach my $parser ( @possible_parsers)
  { if( eval "require $parser")
    { eval "import $parser";# or die "hu-oh, error importing $parser even though the require succeded: $@"; 
      push @parsers, $parser;
    }
  }

unless( @parsers) 
  { die "you need to have a SAX parser installed (one of " . join( ", ", @possible_parsers) . ") none found"; }

BEGIN { use_ok( 'XML::SAX::Context'); }


my @doc;

{ local $/="\n\n";
  @doc= <DATA>;
} 


my %parse_string=( 'XML::Parser::PerlSAX' => 'parse');

foreach my $parser( @parsers)
  { my $parse_string= $parse_string{$parser} || 'parse_string';
    my $test= qq{ sub { my \$doc= shift;
                        #TestHandler\::reset();
                        my \$handler= TestHandler\->new;
                        \$handler->{my_parser_name}= '$parser';
                        my \$parser= $parser\->new(Handler => \$handler);
                        \$parser->$parse_string( \$doc);
                      }
                    };
        #warn "test: $test\n\n";
        my $sub_test= eval "$test";
        foreach my $doc (@doc)
          { $sub_test->( $doc); }
      }

exit;


package TestHandler;

use YAML;
use Test::More;

#sub diag { die @_ };

use base qw(XML::SAX::Context);

# globals so even characters can access them
my $expected_current;
my @expected_context;
my $expected_depth;

sub end_document
  { $expected_current='';
    @expected_context=();
    $expected_depth=0;
  }


sub start_element
  { my( $self, $elt)= @_;
    $expected_current= $elt->{Name};
    push @expected_context, $elt->{Name};
    $expected_depth++;
    is( $self->depth, $expected_depth, "start $expected_current ($expected_depth) - depth") or diag();
    is( $self->current_name, $expected_current, "start $expected_current ($expected_depth) - current_name") or diag();
    is( stringify( $self->ancestor_names), stringify( @expected_context), "ancestor_names (start)") or diag();
  }

sub characters
  { my( $self, $chars)= @_;
    if( $chars->{Data}=~ /pop/)
      { # fake it ! (we have to fake the end_element handler here)
        $expected_depth--;
        pop @expected_context;
        $expected_current= $expected_context[-1];
      }
    is( $self->depth, $expected_depth, "chars in  $expected_current ($expected_depth) - depth")
       or diag( $self->{my_parser_name}, " char: /$chars->{Data}/\ncontext is: \n", Dump( $self->{_context}));
    is( $self->current_name, $expected_current, "chars in  $expected_current ($expected_depth) - current_name")
      or diag( "current_name\nchar: /$chars->{Data}/\ncontext is: \n", Dump( $self->{_context}), "\nexpected_current= $expected_current\ncurrent found: ", $self->current_name, "\n") ;
    is( stringify( $self->ancestor_names), stringify( @expected_context), "ancestor_names (char)") or diag();
  }


sub stringify { return join ':', @_ }

package main;

# one empty line between docs
# add text pop after closing elements so the test can make sense of it

__DATA__
<doc><elt1>text</elt1>[pop]</doc>

<doc>[s1]
  <elt1 id="elt1-1">[s2]
    <elt2 id="elt2-1">This is an elt2 element (1)</elt2>[s3 - pop]
    <elt2 id="elt2-2">This is an elt2 element (2)</elt2>[s4 - pop]
    <elt2 id="elt2-3">This is an elt2 element (3)</elt2>[s5 - pop]
  </elt1>[s6 - pop]
  <elt1 id="elt2-1">[s7]
    <elt2 id="elt2-4">This is an elt2 element (6)</elt2>[s8 - pop]
    <elt2 id="elt2-5">This is an elt2 element (5)</elt2>[s9 - pop]
    <elt2 id="elt2-6">This is an elt2 element (6)</elt2>[s10 - pop]
  </elt1>[s11 - pop]
</doc>

<doc>foo <elt>bar</elt>pop</doc>

<doc>foo <elt />pop</doc>
