# $Id: 03_attributes.t,v 1.3 2003/08/18 13:49:16 mrodrigu Exp $

use strict;

#########################

use Test::More qw(no_plan);

my @possible_parsers= qw( XML::Parser::PerlSAX XML::SAX::PurePerl XML::LibXML::SAX::Parser XML::SAX::Expat );
my @parsers;

my $test_all= -f "test_all";

foreach my $parser ( @possible_parsers)
  { if( eval "require $parser")
    { eval "import $parser";# or die "hu-oh, error importing $parser even though the require succeded: $@"; 
      push @parsers, $parser;
    }
  }

unless( @parsers) 
  { die "you need to have a SAX parser installed (one of " . join( ", ", @possible_parsers) . ") none found"; }

BEGIN { use_ok( 'XML::SAX::Context'); }


my @doc;

{ local $/="\n\n";
  @doc= <DATA>;
} 


my %parse_string=( 'XML::Parser::PerlSAX' => 'parse');

foreach my $parser( @parsers)
  { 
    my $parse_string= $parse_string{$parser} || 'parse_string';
    my $test= qq{ sub { my \$doc= shift;
                        my \$handler= TestHandler\->new;
                        \$handler->{my_parser_name}= '$parser';
                        my \$parser= $parser\->new(Handler => \$handler);
                        \$parser->$parse_string( \$doc);
                      }
                    };
        #warn "test: $test\n\n";
        my $sub_test= eval "$test";
        foreach my $doc (@doc)
          { if( $doc=~ /__INTERNAL__/) { if( $test_all) { next; } else { last; } }
            $sub_test->( $doc);
          }
      }

exit;


package TestHandler;

use YAML;
use Test::More;

use base qw(XML::SAX::Context);

my @atts;

sub start_element { push @atts, $_[1]->{Attributes}; test_atts( $_[0]); }
sub characters    { test_atts( $_[0]); }
sub end_element   { test_atts( $_[0]);  pop @atts; }

sub test_atts
  { my( $self, $elt)= @_;
    foreach my $att (keys %{$atts[-1]})
      { $att=~ s/^\{\}//;
        is(  $self->current_attribute_value( $att), att_simple_val( $self, $att), "testing attribute (simple name)");
        unless( $self->{my_parser_name} eq 'XML::Parser::PerlSAX')
          { is(  $self->current_attribute_value( "{}$att"), att_clark_val( $self, $att), "testing attribute (clarkian)"); }
      }
  }

sub att_simple_val
  { my( $self, $att)= @_;
    if( $self->{my_parser_name} eq 'XML::Parser::PerlSAX')
      { return $atts[-1]->{$att}; }
    else
      { return $atts[-1]->{"{}$att"}->{Value}; }
  }
    
sub att_clark_val
  { my( $self, $att)= @_;
    return $atts[-1]->{"{}$att"}->{Value}; 
  }
    


package main;

# one empty line between docs

__DATA__
<doc att="doc:att" att1="doc:att1" att2="doc:att2"><elt att= "elt:att" att1="elt:att1" att2="elt:att2"/></doc>

<config logdir="/var/log/foo/" debugfile="/tmp/foo.debug">
 <server name="sahara" osname="solaris" osversion="2.6">
   <address>10.0.0.101</address>
   <address>10.0.1.101</address>
 </server>
 <server name="gobi" osname="irix" osversion="6.5">
   <address>10.0.0.102</address>
 </server>
 <server name="kalahari" osname="linux" osversion="2.0.34">
   <address>10.0.0.103</address>
   <address>10.0.1.103</address>
 </server>
</config>


__INTERNAL__

<?xml version="1.0" encoding="ISO-8859-1"?>
<Finvoice Version="1.0">
	<SellerPartyDetails>
		<SellerPartyIdentifier>0123456-7</SellerPartyIdentifier>
		<SellerOrganisationName>Pullin Kala Oy</SellerOrganisationName>
		<SellerOrganisationTaxCode>0123456-7</SellerOrganisationTaxCode>
		<SellerPostalAddressDetails>
			<SellerStreetName>Haapatie 7</SellerStreetName>
			<SellerTownName>Helsinki</SellerTownName>
			<SellerPostCodeIdentifier>00100</SellerPostCodeIdentifier>
			<CountryCode>FI</CountryCode>
			<CountryName>FINLAND</CountryName>
			<SellerPostOfficeBoxIdentifier>PL 302</SellerPostOfficeBoxIdentifier>
		</SellerPostalAddressDetails>
	</SellerPartyDetails>
	<SellerContactPersonName>Hanna Paananen</SellerContactPersonName>
	<SellerCommunicationDetails>
		<SellerPhoneNumberIdentifier>050-5432659</SellerPhoneNumberIdentifier>
		<SellerEmailaddressIdentifier>hanna.paananen@pullinkala.fi</SellerEmailaddressIdentifier>
	</SellerCommunicationDetails>
	<SellerInformationDetails>
		<SellerHomeTownName>Helsinki</SellerHomeTownName>
		<SellerVatRegistrationText>Alv.Rek</SellerVatRegistrationText>
		<SellerVatRegistrationDate Format="CCYYMMDD">19990321</SellerVatRegistrationDate>
		<SellerPhoneNumber>(09) 542 1222</SellerPhoneNumber>
		<SellerFaxNumber>(09) 542 2221</SellerFaxNumber>
		<SellerCommonEmailaddressIdentifier>palaute@pullinkala.fi</SellerCommonEmailaddressIdentifier>
		<SellerWebaddressIdentifier>www.pullinkala.fi</SellerWebaddressIdentifier>
		<SellerAccountDetails>
			<SellerAccountID IdentificationSchemeName="IBAN">FI1234567890123456</SellerAccountID>
			<SellerBic IdentificationSchemeName="BIC">OKOYFIHH</SellerBic>
		</SellerAccountDetails>
		<SellerAccountDetails>
			<SellerAccountID IdentificationSchemeName="IBAN">FI1234567890123456</SellerAccountID>
			<SellerBic IdentificationSchemeName="BIC">NDEAFIHH</SellerBic>
		</SellerAccountDetails>
	</SellerInformationDetails>
	<BuyerPartyDetails>
		<BuyerPartyIdentifier>CL12221</BuyerPartyIdentifier>
		<BuyerOrganisationName>Kalakauppa Vilkkunen</BuyerOrganisationName>
		<BuyerPostalAddressDetails>
			<BuyerStreetName>Haapatie 1</BuyerStreetName>
			<BuyerTownName>Helsinki</BuyerTownName>
			<BuyerPostCodeIdentifier>00211</BuyerPostCodeIdentifier>
			<CountryCode/>
			<CountryName/>
			<BuyerPostOfficeBoxIdentifier/>
		</BuyerPostalAddressDetails>
	</BuyerPartyDetails>
	<BuyerContactPersonName>Maija Vikkunen</BuyerContactPersonName>
	<BuyerCommunicationDetails>
		<BuyerPhoneNumberIdentifier>(09)221222</BuyerPhoneNumberIdentifier>
		<BuyerEmailaddressIdentifier>maija.vilkkunen@kolumbus.fi</BuyerEmailaddressIdentifier>
	</BuyerCommunicationDetails>
	<InvoiceDetails>
		<InvoiceTypeCode>INV01</InvoiceTypeCode>
		<InvoiceTypeText>LASKU</InvoiceTypeText>
		<OriginCode>Origin</OriginCode>
		<InvoiceNumber>1/2002</InvoiceNumber>
		<InvoiceDate Format="CCYYMMDD">20020612</InvoiceDate>
		<SellerReferenceIdentifier>212</SellerReferenceIdentifier>
		<OrderIdentifier>TI0988</OrderIdentifier>
		<InvoiceTotalVatExcludedAmount AmountCurrencyIdentifier="EUR">100,00</InvoiceTotalVatExcludedAmount>
		<InvoiceTotalVatAmount AmountCurrencyIdentifier="EUR">22,00</InvoiceTotalVatAmount>
		<InvoiceTotalVatIncludedAmount AmountCurrencyIdentifier="EUR">122,00</InvoiceTotalVatIncludedAmount>
		<ShortProposedAccountIdentifier>9922</ShortProposedAccountIdentifier>
		<NormalProposedAccountIdentifier>8822</NormalProposedAccountIdentifier>
		<AccountDimensionText>4500</AccountDimensionText>
		<VatSpecificationDetails>
			<VatBaseAmount AmountCurrencyIdentifier="EUR">100,00</VatBaseAmount>
			<VatRatePercent>22</VatRatePercent>
			<VatRateAmount AmountCurrencyIdentifier="EUR">22,00</VatRateAmount>
		</VatSpecificationDetails>
		<PaymentTermsDetails>
			<PaymentTermsFreeText>14 piv netto</PaymentTermsFreeText>
			<PaymentOverDueFineDetails>
				<PaymentOverDueFineFreeText>Yliaikakorko 16%</PaymentOverDueFineFreeText>
				<PaymentOverDueFinePercent>16</PaymentOverDueFinePercent>
			</PaymentOverDueFineDetails>
		</PaymentTermsDetails>
	</InvoiceDetails>
	<PaymentStatusDetails>
		<PaymentStatusCode>NOTPAID</PaymentStatusCode>
	</PaymentStatusDetails>
	<VirtualBankBarcode>250003210002229000122000000000000000862074102062800009</VirtualBankBarcode>
	<InvoiceRow>
		<ArticleIdentifier>123213123332123</ArticleIdentifier>
		<ArticleName>TUURA</ArticleName>
		<BuyerArticleIdentifier>1232321232332</BuyerArticleIdentifier>
		<DeliveredQuantity QuantityUnitCode="Kpl">10</DeliveredQuantity>
		<OrderedQuantity QuantityUnitCode="Kpl">10</OrderedQuantity>
		<UnitPriceAmount AmountCurrencyIdentifier="EUR" UnitPriceUnitCode="e/kpl">5,00</UnitPriceAmount>
		<RowIdentifier>221</RowIdentifier>
		<RowIdentifierDate Format="CCYYMMDD">20020611</RowIdentifierDate>
		<RowDeliveryDate Format="CCYYMMDD">20020615</RowDeliveryDate>
		<RowShortProposedAccountIdentifier>9922</RowShortProposedAccountIdentifier>
		<RowNormalProposedAccountIdentifier>8822</RowNormalProposedAccountIdentifier>
		<RowAccountDimensionText>4500</RowAccountDimensionText>
		<RowFreeText>Tuurat myydn varsien kanssa.</RowFreeText>
		<RowDiscountPercent>0</RowDiscountPercent>
		<RowVatRatePercent>22</RowVatRatePercent>
		<RowVatAmount AmountCurrencyIdentifier="EUR">11,00</RowVatAmount>
		<RowVatExcludedAmount AmountCurrencyIdentifier="EUR">50,00</RowVatExcludedAmount>
		<RowAmount AmountCurrencyIdentifier="EUR">61,00</RowAmount>
	</InvoiceRow>
	<InvoiceRow>
		<ArticleIdentifier>123213123332123</ArticleIdentifier>
		<ArticleName>HUOLLON VUOSIMAKSU</ArticleName>
		<StartDate Format="CCYYMMDDHHMMSS">20020611000000</StartDate>
		<EndDate Format="CCYYMMDDHHMMSS">2003062011000000</EndDate>
		<RowShortProposedAccountIdentifier>9923</RowShortProposedAccountIdentifier>
		<RowNormalProposedAccountIdentifier>8823</RowNormalProposedAccountIdentifier>
		<RowAccountDimensionText>4500</RowAccountDimensionText>
		<RowVatRatePercent>22</RowVatRatePercent>
		<RowVatAmount AmountCurrencyIdentifier="EUR">11,00</RowVatAmount>
		<RowVatExcludedAmount AmountCurrencyIdentifier="EUR">50,00</RowVatExcludedAmount>
		<RowAmount AmountCurrencyIdentifier="EUR">61,00</RowAmount>
	</InvoiceRow>
	<EpiDetails>
		<EpiIdentificationDetails>
			<EpiDate Format="CCYYMMDD">20020612</EpiDate>
		</EpiIdentificationDetails>
		<EpiPartyDetails>
			<EpiBfiPartyDetails>
				<EpiBfiIdentifier IdentificationSchemeName="BIC">OKOYFIHH</EpiBfiIdentifier>
			</EpiBfiPartyDetails>
			<EpiBeneficiaryPartyDetails>
				<EpiNameAddressDetails>Pullin Kala</EpiNameAddressDetails>
				<EpiAccountID IdentificationSchemeName="IBAN">FI1234567890123456</EpiAccountID>
			</EpiBeneficiaryPartyDetails>
		</EpiPartyDetails>
		<EpiPaymentInstructionDetails>
			<EpiRemittanceInfoIdentifier>8620741</EpiRemittanceInfoIdentifier>
			<EpiInstructedAmount AmountCurrencyIdentifier="EUR">122,00</EpiInstructedAmount>
			<EpiCharge ChargeOption="SHA">SHA</EpiCharge>
			<EpiDateOptionDate Format="CCYYMMDD">20020628</EpiDateOptionDate>
		</EpiPaymentInstructionDetails>
	</EpiDetails>
</Finvoice>

