# $Id: 04_inherit_attributes.t,v 1.2 2003/08/18 13:49:16 mrodrigu Exp $

use strict;
use List::Util;

#########################

use Test::More qw(no_plan);

my @possible_parsers= qw( XML::Parser::PerlSAX XML::SAX::PurePerl XML::LibXML::SAX::Parser XML::SAX::Expat );
my @parsers;

foreach my $parser ( @possible_parsers)
  { if( eval "require $parser")
    { eval "import $parser";# or die "hu-oh, error importing $parser even though the require succeded: $@"; 
      push @parsers, $parser;
    }
  }

unless( @parsers) 
  { die "you need to have a SAX parser installed (one of " . join( ", ", @possible_parsers) . ") none found"; }

BEGIN { use_ok( 'XML::SAX::Context'); }


my @doc;

{ local $/="\n\n";
  @doc= <DATA>;
} 


my %parse_string=( 'XML::Parser::PerlSAX' => 'parse');

foreach my $parser( @parsers)
  { 
    my $parse_string= $parse_string{$parser} || 'parse_string';
    my $test= qq{ sub { my \$doc= shift;
                        my \$handler= TestHandler\->new;
                        \$handler->{my_parser_name}= '$parser';
                        my \$parser= $parser\->new(Handler => \$handler);
                        \$parser->$parse_string( \$doc);
                      }
                    };
        #warn "test: $test\n\n";
        my $sub_test= eval "$test";
        foreach my $doc (@doc)
          { $sub_test->( $doc); } 
      }

exit;


package TestHandler;

use YAML;
use Test::More;

use base qw(XML::SAX::Context);

my @atts;

sub start_element { push @atts, $_[1]->{Attributes}; test_atts( $_[0], $_[0]->{my_parser_name}, 'start', $_[1]->{Name}); }
sub characters    { test_atts( $_[0], $_[0]->{my_parser_name}, 'char', $_[1]->{Data}); }
sub end_element   { test_atts( $_[0], $_[0]->{my_parser_name}, 'end', $_[1]->{Name});  pop @atts; }

sub test_atts
  { my( $self, $parser, $step, $data)= @_;
    foreach my $att ( qw( in1 in2 in3) )
      { is(  $self->inherited_att( $att), inherited( $self, $att), "$parser - $step - $data"); }
  }

sub inherited
  { my( $self, $att)= @_;
    foreach my $atts (reverse @atts)
      { if( $self->{my_parser_name} eq 'XML::Parser::PerlSAX')
          { if( my $found= $atts->{$att}) { return $found; } }
        else
          { if( my $found= $atts->{"{}$att"}) { return $found->{Value}; } }
      }
    return undef;
  }
    
package main;

# one empty line between docs

__DATA__
<doc in1="doc_in1"/>

<doc in1="doc_in1">
  <elt in1="elt_in1" in2="elt_in2">
    <sub>in sub</sub>
    <sub in2="sub_in2">in sub 2</sub>
    <sub in3="sub_in3">in sub 3</sub>
    <sub in1="sub_in1">in sub 4</sub>
  </elt>
  <elt in2="elt_in2">
    <sub>in sub</sub>
    <sub in2="sub_in2">in sub 2</sub>
    <sub in3="sub_in3">in sub 3</sub>
    <sub in1="sub_in1">in sub 4</sub>
  </elt>
</doc>
    

